@@ -47,10 +47,10 @@ otamapy_raise(otama_status_t ret)
         case OTAMA_STATUS_OK:
             break;
         case OTAMA_STATUS_NODATA:
-	    case OTAMA_STATUS_INVALID_ARGUMENTS:
-	    case OTAMA_STATUS_ASSERTION_FAILURE:
-	    case OTAMA_STATUS_SYSERROR:
-	    case OTAMA_STATUS_NOT_IMPLEMENTED:
+        case OTAMA_STATUS_INVALID_ARGUMENTS:
+        case OTAMA_STATUS_ASSERTION_FAILURE:
+        case OTAMA_STATUS_SYSERROR:
+        case OTAMA_STATUS_NOT_IMPLEMENTED:
             PyErr_SetString(PyExc_OtamaError, otama_status_message(ret));
             break;
         default:
@@ -75,15 +75,15 @@ variant2pyobj(otama_variant_t *var)
             PyObject *tuple = PyTuple_New(count);
             for (i = 0; i < count; ++i) {
                 PyObject *_value = variant2pyobj(otama_variant_array_at(var, i));
-                PyTuple_SetItem(tuple, i,_value);
+                PyTuple_SetItem(tuple, i, _value);
                 Py_DECREF(_value);
             }
             return tuple;
         }
-	    case OTAMA_VARIANT_TYPE_HASH: {
-		    otama_variant_t *keys = otama_variant_hash_keys(var);
-		    long count = otama_variant_array_count(keys);
-		    int i;
+        case OTAMA_VARIANT_TYPE_HASH: {
+            otama_variant_t *keys = otama_variant_hash_keys(var);
+            long count = otama_variant_array_count(keys);
+            int i;
             PyObject *dict = PyDict_New();
             for (i = 0; i < count; ++i) {
                 PyObject *_value = variant2pyobj(otama_variant_hash_at2(var, otama_variant_array_at(keys, i)));
@@ -95,6 +95,7 @@ variant2pyobj(otama_variant_t *var)
             return dict;
         }
         case OTAMA_VARIANT_TYPE_NULL:
+            Py_RETURN_NONE;
         default:
             printf("not implementation\n");
             break;
@@ -122,10 +123,10 @@ pyobj2variant(PyObject *object, otama_variant_t *var)
 {
     if (PyBool_Check(object)) {
         if (Py_True == object) {
-	        otama_variant_set_int(var, 1);
+            otama_variant_set_int(var, 1);
         }
         else {
-	        otama_variant_set_int(var, 0);
+            otama_variant_set_int(var, 0);
         }
     }
     else if (Py_None == object) {
@@ -151,8 +152,8 @@ pyobj2variant(PyObject *object, otama_variant_t *var)
         PyObject *utf8_item;
         utf8_item = PyUnicode_AsUTF8String(object);
         if (!utf8_item) {
-            // TODO: error handling
-            printf("error\n");
+            PyErr_SetString(PyExc_OtamaError, "don't gen utf8 item");
+            return NULL;
         }
 
         if (strlen(PyBytes_AsString(utf8_item)) == PyUnicode_GET_SIZE(object)) {
@@ -210,10 +211,10 @@ make_results(const otama_result_t *results)
 
     result_tuple = PyTuple_New(num);
     for (i = 0; i < num; ++i) {
-		otama_variant_t *value = otama_result_value(results, i);
+        otama_variant_t *value = otama_result_value(results, i);
         char hexid[OTAMA_ID_HEXSTR_LEN];
 
-		otama_id_bin2hexstr(hexid, otama_result_id(results, i));
+        otama_id_bin2hexstr(hexid, otama_result_id(results, i));
         PyObject *_result = variant2pyobj(value);   // return new dict
         PyObject *_hexid = PyString_FromString(hexid);
         PyDict_SetItemString(_result, "id", _hexid);
@@ -243,8 +244,8 @@ setup_config(OtamaObject *self, PyObject *config)
             PyObject *utf8_item;
             utf8_item = PyUnicode_AsUTF8String(config);
             if (!utf8_item) {
-                // TODO: error handling
-                printf("error\n");
+                PyErr_SetString(PyExc_OtamaError, "don't gen utf8 item");
+                return NULL;
             }
             ret = otama_open(&self->otama, PyBytes_AsString(utf8_item));
             Py_XDECREF(utf8_item);
@@ -346,7 +347,7 @@ OtamaObject_close(OtamaObject *self)
 static PyObject *
 OtamaObject_pull(OtamaObject *self)
 {
-	otama_status_t ret;
+    otama_status_t ret;
 
     ret = otama_pull(self->otama);
     if (ret != OTAMA_STATUS_OK) {
@@ -357,17 +358,56 @@ OtamaObject_pull(OtamaObject *self)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+OtamaObject_create_database(OtamaObject *self)
+{
+    otama_status_t ret;
+
+    if (!self->otama) {
+        PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
+        return NULL;
+    }
+
+    ret = otama_create_database(self->otama);
+    if (ret != OTAMA_STATUS_OK) {
+        otamapy_raise(ret);
+        return NULL;
+    }
+
+    Py_RETURN_NONE;
+}
+static PyObject *
+OtamaObject_drop_database(OtamaObject *self)
+{
+    otama_status_t ret;
+
+    if (!self->otama) {
+        PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
+        return NULL;
+    }
+
+    ret = otama_drop_database(self->otama);
+    if (ret != OTAMA_STATUS_OK) {
+        otamapy_raise(ret);
+        return NULL;
+    }
+
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 OtamaObject_create_table(OtamaObject *self)
 {
     otama_status_t ret;
 
+    fprintf(stderr, "This API is deprecated, rename to create_database\n");
+
     if (!self->otama) {
         PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
         return NULL;
     }
 
-    ret = otama_create_table(self->otama);
+    ret = otama_create_database(self->otama);
     if (ret != OTAMA_STATUS_OK) {
         otamapy_raise(ret);
         return NULL;
@@ -380,12 +420,52 @@ OtamaObject_drop_table(OtamaObject *self)
 {
     otama_status_t ret;
 
+    fprintf(stderr, "This API is deprecated, rename to drop_database\n");
+
     if (!self->otama) {
         PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
         return NULL;
     }
 
-    ret = otama_drop_table(self->otama);
+    ret = otama_drop_database(self->otama);
+    if (ret != OTAMA_STATUS_OK) {
+        otamapy_raise(ret);
+        return NULL;
+    }
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+OtamaObject_drop_index(OtamaObject *self)
+{
+    otama_status_t ret;
+
+    if (!self->otama) {
+        PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
+        return NULL;
+    }
+
+    ret = otama_drop_index(self->otama);
+    if (ret != OTAMA_STATUS_OK) {
+        otamapy_raise(ret);
+        return NULL;
+    }
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+OtamaObject_vacuum_index(OtamaObject *self)
+{
+    otama_status_t ret;
+
+    if (!self->otama) {
+        PyErr_SetString(PyExc_OtamaError, "not initialize/config error");
+        return NULL;
+    }
+
+    ret = otama_vacuum_index(self->otama);
     if (ret != OTAMA_STATUS_OK) {
         otamapy_raise(ret);
         return NULL;
@@ -398,10 +478,10 @@ static PyObject *
 OtamaObject_search(OtamaObject *self, PyObject *args)
 {
     int num;
-	otama_status_t ret;
-	otama_result_t *results = NULL;
-	otama_variant_pool_t *pool;
-	otama_variant_t *var;
+    otama_status_t ret;
+    otama_result_t *results = NULL;
+    otama_variant_pool_t *pool;
+    otama_variant_t *var;
     PyObject *data;
     PyObject *result_tuple;
 
@@ -415,8 +495,8 @@ OtamaObject_search(OtamaObject *self, PyObject *args)
         return NULL;
     }
 
-	pool = otama_variant_pool_alloc();
-	var = otama_variant_new(pool);
+    pool = otama_variant_pool_alloc();
+    var = otama_variant_new(pool);
     pyobj2variant(data, var);
 
     if (PyString_Check(data)) {
@@ -435,8 +515,8 @@ OtamaObject_search(OtamaObject *self, PyObject *args)
         PyObject *utf8_item;
         utf8_item = PyUnicode_AsUTF8String(data);
         if (!utf8_item) {
-            // TODO: error handling
-            printf("error\n");
+            PyErr_SetString(PyExc_OtamaError, "don't gen utf8 item");
+            return NULL;
         }
         const char *_tmp = PyBytes_AsString(utf8_item);
         char _err_tmp[120] = "not exist file ";
@@ -462,8 +542,8 @@ OtamaObject_search(OtamaObject *self, PyObject *args)
     }
     result_tuple = make_results(results);
 
-	otama_result_free(&results);
-	otama_variant_pool_free(&pool);
+    otama_result_free(&results);
+    otama_variant_pool_free(&pool);
 
     return result_tuple;
 }
@@ -494,25 +574,25 @@ OtamaObject_similarity(OtamaObject *self, PyObject *args)
     pyobj2variant(data1, var1);
     pyobj2variant(data2, var2);
 
-	ret = otama_similarity(self->otama, &similarity, var1, var2);
-	if (ret != OTAMA_STATUS_OK) {
-		otama_variant_pool_free(&pool);
-		otamapy_raise(ret);
-		return NULL;
-	}
-	otama_variant_pool_free(&pool);
+    ret = otama_similarity(self->otama, &similarity, var1, var2);
+    if (ret != OTAMA_STATUS_OK) {
+        otama_variant_pool_free(&pool);
+        otamapy_raise(ret);
+        return NULL;
+    }
+    otama_variant_pool_free(&pool);
 
-	return PyFloat_FromDouble(similarity);
+    return PyFloat_FromDouble(similarity);
 }
 
 static PyObject *
 OtamaObject_insert(OtamaObject *self, PyObject *args)
 {
-	char hexid[OTAMA_ID_HEXSTR_LEN];
-	otama_id_t id;
-	otama_status_t ret;
-	otama_variant_pool_t *pool;
-	otama_variant_t *var;
+    char hexid[OTAMA_ID_HEXSTR_LEN];
+    otama_id_t id;
+    otama_status_t ret;
+    otama_variant_pool_t *pool;
+    otama_variant_t *var;
     PyObject *data;
     PyObject *pyobj_id;
 
@@ -521,9 +601,9 @@ OtamaObject_insert(OtamaObject *self, PyObject *args)
         return NULL;
     }
 
-	pool = otama_variant_pool_alloc();
-	var = otama_variant_new(pool);
-	pyobj2variant(data, var);
+    pool = otama_variant_pool_alloc();
+    var = otama_variant_new(pool);
+    pyobj2variant(data, var);
 
     if (PyString_Check(data)) {
         const char *_tmp = PyString_AsString(data);
@@ -533,8 +613,8 @@ OtamaObject_insert(OtamaObject *self, PyObject *args)
         PyObject *utf8_item;
         utf8_item = PyUnicode_AsUTF8String(data);
         if (!utf8_item) {
-            // TODO: error handling
-            printf("error\n");
+            PyErr_SetString(PyExc_OtamaError, "don't gen utf8 item");
+            return NULL;
         }
         const char *_tmp = PyBytes_AsString(utf8_item);
         ret = otama_insert_file(self->otama, &id, _tmp);
@@ -551,8 +631,8 @@ OtamaObject_insert(OtamaObject *self, PyObject *args)
         return NULL;
     }
 
-	otama_id_bin2hexstr(hexid, &id);
-	otama_variant_pool_free(&pool);
+    otama_id_bin2hexstr(hexid, &id);
+    otama_variant_pool_free(&pool);
 
     pyobj_id = Py_BuildValue("s", hexid);
     return pyobj_id;
@@ -620,6 +700,54 @@ OtamaObject_exists(OtamaObject *self, PyObject *args)
     return result == 0 ? PyBool_FromLong(0) : PyBool_FromLong(1);
 }
 
+static PyObject *
+OtamaObject_invoke(OtamaObject *self, PyObject *args)
+{
+    const char *_tmp_method;
+    PyObject *output, *method, *input;
+    otama_status_t ret;
+    otama_variant_pool_t *pool = otama_variant_pool_alloc();
+    otama_variant_t *input_var = otama_variant_new(pool);
+    otama_variant_t *output_var = otama_variant_new(pool);
+
+    if (!PyArg_ParseTuple(args, "OO", &method, &input)) {
+        PyErr_SetString(PyExc_TypeError, "argument error");
+        return NULL;
+    }
+
+    if (PyString_Check(method)) {
+        _tmp_method = PyString_AsString(method);
+    }
+    else if (PyUnicode_Check(method)) {
+        PyObject *utf8_item;
+        utf8_item = PyUnicode_AsUTF8String(method);
+        if (!utf8_item) {
+            PyErr_SetString(PyExc_OtamaError, "don't gen utf8 item");
+            return NULL;
+        }
+        _tmp_method = PyBytes_AsString(utf8_item);
+        Py_XDECREF(utf8_item);
+    }
+    else {
+        PyErr_SetString(PyExc_TypeError, "not support type");
+        return NULL;
+    }
+
+    pyobj2variant(input, input_var);
+
+    ret = otama_invoke(self->otama, _tmp_method, output_var, input_var);
+    if (ret != OTAMA_STATUS_OK) {
+        otama_variant_pool_free(&pool);
+        otamapy_raise(ret);
+        return NULL;
+    }
+
+    output = variant2pyobj(output_var);
+    otama_variant_pool_free(&pool);
+
+    return output;
+}
+
 static PyObject *
 OtamaObject_feature_raw(OtamaObject *self, PyObject *args)
 {
@@ -658,6 +786,44 @@ OtamaObject_feature_raw(OtamaObject *self, PyObject *args)
     return pyraw;
 }
 
+static PyObject *
+OtamaObject_feature_string(OtamaObject *self, PyObject *args)
+{
+    otama_status_t ret;
+    PyObject *pystr, *query;
+    otama_variant_pool_t *pool;
+    otama_variant_t *var;
+    char *feature_string = NULL;
+
+    if (!PyArg_ParseTuple(args, "O", &query)) {
+        PyErr_SetString(PyExc_TypeError, "argument error");
+        return NULL;
+    }
+
+    if (!PyDict_Check(query)) {
+        PyErr_SetString(PyExc_TypeError, "invalid argument");
+        return NULL;
+    }
+
+    pool = otama_variant_pool_alloc();
+    var = otama_variant_new(pool);
+
+    pyobj2variant(query, var);
+
+    ret = otama_feature_string(self->otama, &feature_string, var);
+    if (ret != OTAMA_STATUS_OK) {
+        otama_variant_pool_free(&pool);
+        otamapy_raise(ret);
+        return NULL;
+    }
+    otama_variant_pool_free(&pool);
+
+    pystr = PyString_FromString(feature_string);
+    // TODO: error handling
+
+    return pystr;
+}
+
 static PyObject *
 OtamaFeatureRawObject_dispose(OtamaFeatureRawObject *self)
 {
@@ -673,23 +839,35 @@ static PyMethodDef OtamaObject_methods[] = {
     {"close", (PyCFunction)OtamaObject_close, METH_NOARGS,
      "close Otama Object"},
     {"pull", (PyCFunction)OtamaObject_pull, METH_NOARGS,
-     "pull to Otama DataBase"},
+     "pull to Otama Database"},
+    {"create_database", (PyCFunction)OtamaObject_create_database, METH_NOARGS,
+     "create Otama Database Table"},
+    {"drop_database", (PyCFunction)OtamaObject_drop_database, METH_NOARGS,
+     "drop to Otama Database Table"},
     {"create_table", (PyCFunction)OtamaObject_create_table, METH_NOARGS,
-     "create Otama DataBase Table"},
+     "create Otama Database Table (deprecated)"},
     {"drop_table", (PyCFunction)OtamaObject_drop_table, METH_NOARGS,
-     "drop to Otama DataBase Table"},
+     "drop to Otama Database Table (deprecated)"},
+    {"drop_index", (PyCFunction)OtamaObject_drop_index, METH_NOARGS,
+     "drop to Otama Database Index"},
+    {"vacuum_index", (PyCFunction)OtamaObject_vacuum_index, METH_NOARGS,
+     "vacuum to Otama Database Index"},
     {"insert", (PyCFunction)OtamaObject_insert, METH_VARARGS,
      "insert image data"},
     {"remove", (PyCFunction)OtamaObject_remove, METH_VARARGS,
-     "remove id from Otama DataBase"},
+     "remove id from Otama Database"},
     {"search", (PyCFunction)OtamaObject_search, METH_VARARGS,
-     "search from Otama DataBase"},
+     "search from Otama Database"},
     {"similarity", (PyCFunction)OtamaObject_similarity, METH_VARARGS,
      "check similarity"},
     {"exists", (PyCFunction)OtamaObject_exists, METH_VARARGS,
-     "exist image in Otama DataBase"},
+     "exist image in Otama Database"},
+    {"feature_string", (PyCFunction)OtamaObject_feature_string, METH_VARARGS,
+     "return feature string value"},
     {"feature_raw", (PyCFunction)OtamaObject_feature_raw, METH_VARARGS,
      "return feature raw value"},
+    {"invoke", (PyCFunction)OtamaObject_invoke, METH_VARARGS,
+     "invoke Database driver"},
     {NULL, NULL, 0, NULL}
 };
 
@@ -815,7 +993,8 @@ static struct PyModuleDef OtamaModuleDef = {
     -1,
     OtamaMethods,
 };
-PyObject *
+
+PyMODINIT_FUNC
 PyInit_otama(void)
 
 #else
@@ -856,6 +1035,7 @@ initotama(void)
     Py_INCREF(PyExc_OtamaError);
     PyModule_AddObject(module, "error", (PyObject *)&OtamaObjectType);
 
+    otama_log_set_level(OTAMA_LOG_LEVEL_ERROR);
 #ifdef PY3
     return module;
 #endif
